/***********************************************************************************/
/**	\file				Receiver.cpp
**	\brief				Implementation zur Klasse Receiver
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		03.04.2002
**	letzte Aenderung:	03.04.2002
*************************************************************************************
**	Diese Klasse ist fr das Empfangen und zusammensetzen der Nachrichten zustndig.
************************************************************************************/


// includes
#include "Receiver.h"

// Methoden-Definitionen

/************************************************************************************
**	Methode:			Receiver::Receiver
************************************************************************************/
/**	Konstruktor, Port merken
***
************************************************************************************/
	Receiver::Receiver(int iPort)
{
		m_iPort=iPort;
}

/************************************************************************************
**	Methode:			Receiver::~Receiver
************************************************************************************/
/**	Destruktor, Vektorinhalte lschen
***
************************************************************************************/
	Receiver::~Receiver()
{
	showFrags(m_pmcfvecNew,false);
	showFrags(m_pmcfvecOrdered,false);
}

/************************************************************************************
**	Methode:			Receiver::run
************************************************************************************/
/**	Diese Methode lsst das Empfangen ablaufen.
************************************************************************************/
void Receiver::run(void)
{
	RNPRESULT rnprTmp;
	RNPacket rnpRecv;
	RNSocket rnsListener;

	// MessageHandler initialisieren
	if(rnpRecv.initMessageCallBack(this,StaticMessageHandler) == RNP_OK)
	{
		printf("MessageHandler inited.\n");
	}
	else
	{
		return;
	}

	// Socket zum lauschen erzeugen
	if (!rnsListener.create())
	{
		printf("Could not create Socket to listen.\n");
		return;
	}

	// Socket an Port binden
	if (!rnsListener.bind(NULL,m_iPort))
	{
		printf("Could not bind Socket to listen.\n");
		
		return;
	}

	// Start des Lauschens
	if (!rnsListener.listen())
	{
		printf("Could not listen, Socket Error.\n");
		return;
	}

	printf("Listening...");

	// Abwarten, bist jemand einen Connect auf den Port versucht
	do
	{
		rnprTmp=rnpRecv.socketAccept(&rnsListener);

		if (kbhit())
		{
			printf("\nUser aborted.\n");
			getch();
			return;
		}
		
		Sleep(100);
		printf(".");
	} while (rnprTmp == RNP_SOCKET_NOT_CONNECTED);

	printf("\n");

	// berprfen, ob Verbindung erfolgreich zustande kam
	if (rnprTmp != RNP_OK)
	{
		printf("Error occured.\n");
		return;
	}

	// es hat funktioniert!
	printf("Connection established.\n");
	
	// empfangen bis disonnection auftritt
	do
	{
		rnprTmp=rnpRecv.recvMessage();

		if (kbhit())
		{
			printf("User aborted.\n");
			getch();
			return;
		}
	} while (rnprTmp != RNP_SOCKET_NOT_CONNECTED);

	printf("Connection closed.\n");

	// eigenen Socket auch schliessen
	if(rnpRecv.socketClose() == RNP_OK)
	{
		printf("Socket closed.\n");
	}
	else
	{
		printf("Error occured while closing Socket.\n");
	}
}

/************************************************************************************
**	Methode:			Receiver::findPacketNr
************************************************************************************/
/**	Sucht in einen Vektor aus CharFragmenten, nach einem Packet mit einer bestimmten
***	PacketNr. Dessen Position wird zurckgegeben, wird keines gefunden, wird -1 
***	zurckgegeben.
************************************************************************************/
int		Receiver::findPacketNr(FragVec &pmcfvecSrc, DWORD dwPacketNr)
{
	int iCount;
			
	for (iCount=0; iCount<pmcfvecSrc.size();iCount++)
	{
		// hat dieses Packet die Nummer?
		if (pmcfvecSrc[iCount]->m_dwPacketNr==dwPacketNr)
		{
			// ja, Nummer zurck
			return iCount;
		}
	}

	// nicht gefunden, -1 zurck
	return -1;
}

/************************************************************************************
**	Methode:			Receiver::showfrags
************************************************************************************/
/**	geht die Fragmente des Vektors alle durch und lscht diese. Falls bShow true
***	sein sollte, werden die Textstcke angezeigt.
************************************************************************************/
void Receiver::showFrags(FragVec &pmcfvecSrc, bool bShow/* =true */)
{
	MessageCharFragment* pmcfTmp;
	char pcBuffer[c_dwSizeCharFragment+1];
	int iCount=0;
			
	while (pmcfvecSrc.size()>0)
	{
		pmcfTmp=*(pmcfvecSrc.begin());

		if (pmcfTmp != NULL)
		{
			if (bShow==true)
			{
				strncpy(pcBuffer,pmcfTmp->m_cBuffer,c_dwSizeCharFragment);
				pcBuffer[c_dwSizeCharFragment]=0;
				printf("%s",pcBuffer);
				iCount++;
			}
			
			delete pmcfTmp;		
		}

		pmcfvecSrc.erase(pmcfvecSrc.begin());
	}

	if (iCount>0)
	{
		printf("\n%i Fragments showed.\n",iCount);
	}
}

/************************************************************************************
**	Methode:			Receiver::StaticMessageHandler
************************************************************************************/
/**	Ruft den eigentlichen (nicht static definierten) MessageHandler auf.
***
************************************************************************************/
RNPRESULT Receiver::StaticMessageHandler(	void* pvoidUserContext, 
											void* pvoidPacket, 
											WORD wSize)
{
	Receiver* pThisObject = (Receiver*) pvoidUserContext;

    return pThisObject->MessageHandler(pvoidPacket,wSize);
}

/************************************************************************************
**	Methode:			Receiver::MessageHandler
************************************************************************************/
/**	Empfngt und verarbeitet die Nachrichten. Dafr wird zunchst der Nachrichtentyp
*** anhand von MessageGeneric::m_Type erkannt, und dann der void* entsprechend
***	gecasted.
************************************************************************************/
RNPRESULT Receiver::MessageHandler(void* pvoidPacket, WORD wSize)
{
	MessageGeneric* pmsgTmp= (MessageGeneric*)pvoidPacket;

	switch (pmsgTmp->m_Type)
	{
	case MSGID_CHARFRAGMENT:
		{
			MessageCharFragment* pmsgcfTmp= (MessageCharFragment*)pmsgTmp;
			m_pmcfvecNew.push_back(pmsgcfTmp);
			
			int iPos;
			DWORD dwFind=c_dwFirstCharFragment;

			do
			{
				if (m_pmcfvecOrdered.size()>0)
				{
					dwFind=(*(m_pmcfvecOrdered.end()-1))->m_dwNextPacketNr;
				}

				iPos=findPacketNr(m_pmcfvecNew,dwFind);

				if (iPos!=-1)
				{
					m_pmcfvecOrdered.push_back(m_pmcfvecNew[iPos]);
					m_pmcfvecNew.erase(m_pmcfvecNew.begin()+iPos);
				}

			} while (iPos!=-1);

			if ((*(m_pmcfvecOrdered.end()-1))->m_dwNextPacketNr==c_dwLastCharFragment)
			{
				showFrags(m_pmcfvecOrdered);
			}
		}
		return RNP_BUFFER_IN_USE;
		break;

	default:
		return RNP_BUFFER_RETURNED;
	}

	return RNP_BUFFER_RETURNED;
}

/************************************************************************************
**	Ende der Datei:		Receiver.cpp
************************************************************************************/
