/***********************************************************************************/
/**	\file				Sender.cpp
**	\brief				Implementation zur Klasse Sender
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		04.04.2002
**	letzte Aenderung:	04.04.2002
*************************************************************************************
**	Diese Klasse ist fr das Zerschneiden und Verschicken der Nachrichten zustndig.
************************************************************************************/


// includes
#include "Sender.h"

// Methoden-Definitionen

/************************************************************************************
**	Methode:			Sender::Sender
************************************************************************************/
/**	Konstruktor, hinterlegt den Port
************************************************************************************/
Sender::Sender(	int iPort)
{
	m_iPort=iPort;
}

/************************************************************************************
**	Methode:			Sender::run
************************************************************************************/
/**	Fragt den Users nach einer IP und baut eine Verbindung auf. Der User kann nun
*** Textfiles verschicken.
************************************************************************************/
void Sender::run(void)
{
	RNPRESULT rnprTmp;
	char pcEingabe[1024];

	// IP erfragen
	printf("IP >");
	gets(pcEingabe);

	// RNPacket initialisieren
	RNPacket rnpSend;

	if(rnpSend.initMessageCallBack(this,StaticMessageHandler) == RNP_OK)
	{
		printf("MessageHandler inited.\n");
	}
	else
	{
		return;
	}

	// Verbindung aufbauen
	printf("Trying to connect...");
	
	do
	{
		rnprTmp=rnpSend.socketConnect(pcEingabe,m_iPort);
		
		if (kbhit())
		{
			printf("\nUser aborted.\n");
			getch();
			return;
		}
		
		Sleep(100);
		printf(".");
	} while (rnprTmp == RNP_SOCKET_NOT_CONNECTED);
	
	printf("\n");

	// Testen ob Verbindung erfolgreich
	if (rnprTmp != RNP_OK)
	{
		printf("Error occured.\n");
		return;
	}
	
	printf("Connected to %s.\n",pcEingabe);

	DWORD dwPacketNr=0;
	int iCharCount;

	// In Schleife nach Filenamen fragen
	while(true)
	{
		printf("File >");

		gets(pcEingabe);

		// Bei leerer Eingabe beenden
		if (pcEingabe[0]==0)
		{
			break;
		}

		// File ffnen
		FILE* pfileIn=fopen(pcEingabe,"r");
		iCharCount=0;

		// Erfolgreich geffnet ?
		if (pfileIn==NULL)
		{
			printf("File does not exist! Sending input itself.\n");
		}
		else
		{
			printf("Sending file...\n");
		}

		// Packet erstellen
		MessageCharFragment msgcfTmp;
		msgcfTmp.m_Type=MSGID_CHARFRAGMENT;
		msgcfTmp.m_dwNextPacketNr=c_dwFirstCharFragment;

		int iBuf;

		// Packete in einer Schleife erstellen
		do 
		{
			// dieses Packet bekommt Nummer, die das Vorhergehende als Nachfolger hatte (klar, oder? ;)
			msgcfTmp.m_dwPacketNr=msgcfTmp.m_dwNextPacketNr;

			int iCount;

			// Charbuffer lesen aus File oder Eingabe
			for (iCount=0; iCount<c_dwSizeCharFragment; iCount++)
			{
				if (pfileIn!=NULL)
				{
					iBuf=fgetc(pfileIn);
				}
				else
				{
					iBuf=pcEingabe[iCharCount];

					if (iBuf==0)
					{
						iBuf=EOF;
					}
					else
					{
						iCharCount++;
					}
				}
				
				msgcfTmp.m_cBuffer[iCount]=(char)iBuf;

				// am Ende nur noch Nullen anhngen
				if (iBuf==EOF)
				{
					msgcfTmp.m_cBuffer[iCount]=0;
				}
			}

			// Packetnummer weiterzhlen
			if (iBuf==EOF)
			{
				msgcfTmp.m_dwNextPacketNr=c_dwLastCharFragment;
			}
			else
			{
				msgcfTmp.m_dwNextPacketNr=dwPacketNr;
				dwPacketNr++;
			}

			// Senden, falls mglich
			do
			{
				rnprTmp=rnpSend.sendMessage((void*)&msgcfTmp,sizeof(msgcfTmp));

				// bei Tastedruck abbrechen
				if (kbhit())
				{
					break;
				}

			} while (rnprTmp==RNP_SOCKET_NOT_WRITEABLE);
			
			//Verbindungsabbruch
			if (rnprTmp != RNP_OK)
			{
				printf("Connection closed...\n");
				return;
			}

			//User hat gedrckt
			if (kbhit())
			{
				printf("\nUser aborted.\n");
				getch();
				return;
			}			

		} while (iBuf!=EOF);

		// Alle Pakete erfolgreich versendet! (evtl. File schliessen)
		printf("All Packets send.\n");
		if (pfileIn != NULL)
			fclose(pfileIn);
	}
	
	// Socket schliessen
	if(rnpSend.socketClose() == RNP_OK)
	{
		printf("Socket closed.\n");
	}
	else
	{
		printf("Error occured while closing Socket.\n");
	}
	
}

/************************************************************************************
**	Methode:			Sender::StaticMessageHandler
************************************************************************************/
/**	Ruft den eigentlichen (nicht static definierten) MessageHandler auf.
***
************************************************************************************/
RNPRESULT Sender::StaticMessageHandler(		void* pvoidUserContext, 
											void* pvoidPacket, 
											WORD wSize)
{
	Sender* pThisObject = (Sender*) pvoidUserContext;

    return pThisObject->MessageHandler(pvoidPacket,wSize);
}

/************************************************************************************
**	Methode:			Sender::MessageHandler
************************************************************************************/
/**	Empfngt und verarbeitet die Nachrichten. Dafr wird zunchst der Nachrichtentyp
*** anhand von MessageGeneric::m_Type erkannt, und dann der void* entsprechend
***	gecasted.
************************************************************************************/
RNPRESULT Sender::MessageHandler(void* pvoidPacket, WORD wSize)
{
	MessageGeneric* pmsgTmp= (MessageGeneric*)pvoidPacket;
/*
	switch (pmsgTmp->m_Type)
	{
	default:
		return RNP_BUFFER_RETURNED;
	}
*/
	return RNP_BUFFER_RETURNED;
}

/************************************************************************************
**	Ende der Datei:		Sender.cpp
************************************************************************************/
