/***********************************************************************************/
/**	\file				MessageGeneric.h
**	\brief				Header File zur Struktur MessageGeneric
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		27.03.2002
**	letzte Aenderung:	01.04.2002
*************************************************************************************
**	Das File enthlt den Aufbau der Struktur MessageGeneric und der davon
**	abgeleiteten Strukturen
************************************************************************************/

#ifndef _MessageGeneric_H
#define _MessageGeneric_H

// includes
#include <windows.h>

/***********************************************************************************/
/** Enumeration aller Messagetypen
************************************************************************************/
enum MessageType
{
	//! koresspondiert zu MessageCharFragment
	MSGID_CHARFRAGMENT,		 
};

// Compiler alignment ndern, damit kein Platz verschwendet wird
#pragma pack( push, 1 )

/***********************************************************************************/
/** \brief	Der allgemeine Messagetyp
*************************************************************************************
**	Dies ist nur eine Vorlage fr alle "echten" Messagetypen, damit sichergestellt
**  ist, das jede den Typ enthlt und die Messages unterscheidbar sind.
************************************************************************************/

struct	MessageGeneric
{
	//! Typ der Message
	MessageType		m_Type;		
};



//! globale Variable mit dem Wert fr "erstes Paket"
const DWORD c_dwFirstCharFragment= 0xFFFFFFFE;
//! globale Variable mit dem Wert fr "letztes Paket"
const DWORD c_dwLastCharFragment= 0xFFFFFFFF;
//! Gibt die Lnge eines Stcks der Zeichenkette an
const DWORD c_dwSizeCharFragment= 64;

/***********************************************************************************/
/** \brief	Fragmente von Zeichenketten
*************************************************************************************
**	Mit dieser Message knnen Teile von Zeichenketten verschickt werden. Mit Hilfe 
**  der PaketNummern knnen diese wieder zusammengesetzt werden.
************************************************************************************/

struct	MessageCharFragment : public MessageGeneric
{
	//! Nummer dieses Paketes
	DWORD	m_dwPacketNr;
	//! Teil der Zeichenkette
	char	m_cBuffer[c_dwSizeCharFragment];
	//! Nummer des folgenden Paketes
	DWORD	m_dwNextPacketNr;	
};

#pragma pack( pop )

#endif//_MessageGeneric_H

enum MessageType;

/************************************************************************************
**	Ende der Datei:		MessageGeneric.h
************************************************************************************/
