/***********************************************************************************/
/**	\file				RNPacket.h
**	\brief				Header File zur Klasse RNPacket
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		27.03.2002
**	letzte Aenderung:	01.04.2002
*************************************************************************************
**	Die Klasse RNPacket sendet oder empfngt Daten in der Form von beliebigen
**  Datenblcken.
************************************************************************************/

#ifndef _RNPacket_H
#define _RNPacket_H

// includes
#include "..\common\RNSocket.h"

/***********************************************************************************/
/** Enumeration der Rckgabewerte in RNPacket
************************************************************************************/
enum RNPRESULT
{
	//! Daten gesendet
	RNP_OK,		 
	//! Buffer auf das Packet wird nicht mehr bentigt
	RNP_BUFFER_RETURNED,		 
	//! Buffer auf das Packet wird benutzt und spter selbst deleted
	RNP_BUFFER_IN_USE,		 
	//! Paramter sind invalid
	RNP_ERROR_INVALID_PARAM = 0x80000000,		 
	//! RNPacket ist noch nicht initialisiert
	RNP_ERROR_NOT_INITIALIZED,		 
	//! es ist ein Fehler bei Zugriff auf den Socket aufgetreten
	RNP_ERROR_SOCKET,		 
	//! ein undefinierter Fehler ist aufgetreten
	RNP_ERROR_UNDEFINED,		 
	//! Socket konnte nicht beschrieben werden
	RNP_SOCKET_NOT_WRITEABLE,		 
	//! Socket konnte nicht gelesen werden
	RNP_SOCKET_NOT_READABLE,	 
	//! Socket konnte nicht verbunden werden
	RNP_SOCKET_NOT_CONNECTED,	 
};

//!	maximale Paketgrsse
const int c_iMaxPacketLength=2+65536;

/***********************************************************************************/
/** \brief	Der Funktiontyp des MessageHandler CallBacks
*************************************************************************************
**	Um RNPacket zu benutzen, ist eine CallBack Funktion von diesem Typ ntig.
**	pvoidUserContext ist der Pointer, der bei RNPacket::initMessageCallBack
**	bergeben wurde, pvoidPacket ist der Pointer auf das Paket. Die Funktion
**	muss RNP_BUFFER_RETURNED oder RNP_BUFFER_IN_USE zurckgeben.
************************************************************************************/
typedef RNPRESULT (*PMESSAGECALLBACK)( 
  void* pvoidUserContext,
  void* pvoidPacket,
  WORD	wSize
);

// Klassen-Deklaration

/***********************************************************************************/
/** \brief	RNPacket sendet und empfngt Pakete
*************************************************************************************
**	RNPacket kmmert sich darum, das beim Senden automatisch die Lngeninformation
**	dem Paket hinzugefgt wird und diese beim Empfangen wieder entfernt wird. 
************************************************************************************/

class	RNPacket
{
public:
	RNPacket(void);
	~RNPacket(void);
	
RNPRESULT	initMessageCallBack(void* pvoidUserContext,const PMESSAGECALLBACK pmcFoo);
	
RNPRESULT	sendMessage(void* pvoidData, WORD wBufferSize);
	
RNPRESULT	recvMessage(void);

RNPRESULT	socketConnect(	char* pcDestName,
							int iPort,
							int iAdressFamily = AF_INET , 
							int iType = SOCK_STREAM,
							int iProtocol = IPPROTO_IP);

RNPRESULT	socketAccept(RNSocket* psocketListener);

RNPRESULT	socketClose(void);

protected:

private:

	//! Der Socket ber den gesendet oder empfangen wird
	RNSocket			m_RNSSocket;

	//! gibt an ob Objekt bereits initialisiert wurde
	bool				m_bInitialized;

	//! Funktionspointer auf den Messagehandler CallBack
	PMESSAGECALLBACK	m_pmcCallBack;

	//! Der User Kontext, der bei initialize bergeben wurde
	void*				m_pvoidUserContext;

	//! Buffer fr teilweise empfangene Nachrichten
	char				m_pcBuffer[c_iMaxPacketLength];

	//! Anzahl der Bytes, die noch im Buffer sind
	int					m_iBufferUsage;
};

#endif//_RNPacket_H

class	RNPacket;
enum	RNPRESULT;

/************************************************************************************
**	Ende der Datei:		RNPacket.h
************************************************************************************/
