/***********************************************************************************/
/**	\file				RNSocket.h
**	\brief				Header File zur Klasse RNSocket
*************************************************************************************
**	Autor:				Christian Roesch
**
**	Erstelldatum:		27.03.2002
**	letzte Aenderung:	31.03.2002
*************************************************************************************
**	Die Klasse RNSocket beitet eine Klasse zur Kapselung von WinSock.
************************************************************************************/

#ifndef _RNSocket_H
#define _RNSocket_H

// includes
#include <winsock.h>
#include <stdio.h>

// Klassen-Deklaration

/***********************************************************************************/
/** \brief	Kapsel fr WinSock
*************************************************************************************
**	Die Klasse RNSocket beitet eine Klasse zur Kapselung von WinSock. Sie enthlt 
**	Methoden um Verbindungen herzustellen und Daten zu senden und zu empfangen.
**  Die Klasse gibt jeweils bool Werte zurck, die den Erfolg der Methoden angeben.
**	Bei einem false ist die Methode gescheitert, und der entsprechende Fehlercode
**	kann ber WSAGetLastError ausgelesen werden.
************************************************************************************/

class	RNSocket
{
public:
	RNSocket(void);
	~RNSocket(void);

static bool	init(int iMajorVersion = 1, int iMinorVersion = 1);

static bool	cleanUp(void);

bool	create(int iAdressFamily=AF_INET, int iType=SOCK_STREAM,int iProtocol=IPPROTO_IP);

bool	close(void);

bool	shutdown(int iFlags);

bool	connect(char* pcDestName, int iPort, int iAddressFamily=AF_INET);

bool	bind(char* pcDestName, int iPort);

bool	listen(int iBackLog=SOMAXCONN);

bool	accept(RNSocket &socketNew);

bool	receive(void* pvoidData, int* piLength, int iFlags=0);

bool	send(void* pvoidData, int* piLength, int iFlags=0);

bool	isReadable(bool &bReadable, int iMilliSeconds);

bool	isWriteable(bool &bWriteable, int iMilliSeconds);

protected:

private:
	//! Socket - Handle 
	SOCKET		m_Socket;
	
	//! Adresse dieses Sockets
	sockaddr_in	m_addrLocal;

	//! Adresse des Ziel-Sockets
	sockaddr_in m_addrTarget;

	//! Socket Version
	static WSAData	m_wsaData;

	//! gibt an, um WinSockets initialisiert sind
	static bool	m_bSocketInitialized;
};

#endif//_RNSocket_H

class	RNSocket;


/************************************************************************************
**	Ende der Datei:		RNSocket.h
************************************************************************************/
